#include "stdafx.h"
using namespace std;

#ifdef _UNICODE 
  wostream& tcerr = wcerr;
  wostream& tcout = wcout; 
  typedef wifstream tifstream;
  typedef wstring tstring;

  wostream &FixLength(wostream &stream) {
    stream.setf(ios::left);
    stream << setw(6) << setfill(_T(''));
    return stream;
  }
#else 
  ostream& tcerr = cerr; 
  ostream& tcout = cout; 
  typedef ifstream tifstream;
  typedef string tstring;

  ostream &FixLength(ostream &stream) {
    stream.setf(ios::left);
    stream << setw(12) << setfill(_T('*'));
    return stream;
  }
#endif

const int MAX_CHAR = 100;
const enum ErrorType {
  SUCCESS, 
  ERROR_ARGUMENT_NUMBER, 
  ERROR_ARGUMENT_TYPE,
  ERROR_CANNOT_OPEN_FILE
};

const tstring ErrorMessage[4] = {
  _T(""),
  _T("R}hČႢ܂"),
  _T("R}hCsK؂ł"),
  _T("t@CJ܂ł")
};

typedef struct {
  tstring Kanji;
  tstring Hiragana;
  bool IsMale;
} Name;

void ShowVersion();
ErrorType MakeName(int howMuchData);
int GetRandomNumber(int max);

int _tmain(int argc, _TCHAR* argv[]) {
  ErrorType result = SUCCESS;
  _tsetlocale(LC_ALL, _T(""));

  if ((argc < 2) || (argc > 3)) {
    result = ERROR_ARGUMENT_NUMBER;
  } else {
    if (argv[1][0] == _T('-')) {
      switch (argv[1][1]) {
        case _T('v'):
          ShowVersion();
          break;
        default:
          result = ERROR_ARGUMENT_TYPE;
      }
    } else {
      result = MakeName(_tstoi(argv[1]));
    }
  }
  if (result != SUCCESS)
    tcerr << ErrorMessage[result] << endl;
  return result;
}

void ShowVersion() {
  _TCHAR mode = _T('s');
  #ifdef _MBCS
    mode = _T('m');
  #endif
  #ifdef _UNICODE
    mode = _T('u');
  #endif
  tcout << _T("Test Data Generator Version 0.03") 
    << mode << _T(" (c)2009 Hideo Harada\n");  
  return;
}

ErrorType MakeName(int howMuchData) {
  ErrorType result = SUCCESS;
  vector<Name> lastName, firstName;
  TCHAR s[MAX_CHAR];
  Name n;
  const TCHAR delimiter[] = _T(",");
  TCHAR *next = NULL;

  tifstream inLastName(_T("lastname.txt"));
  if (!inLastName) return ERROR_CANNOT_OPEN_FILE; 
  inLastName.imbue(locale("japanese"));
  while (inLastName) {
    inLastName.getline(s, MAX_CHAR);
    if (inLastName) {
      n.Kanji = _tcstok_s(s, delimiter, &next);
      n.Hiragana = _tcstok_s(next, delimiter, &next);
      n.IsMale = true;
      lastName.push_back(n);
    }
  }
  inLastName.close();

  tifstream inFirstName(_T("firstname.txt"));
  if (!inFirstName) return ERROR_CANNOT_OPEN_FILE; 
  inFirstName.imbue(locale("japanese"));
  while (inFirstName) {
    inFirstName.getline(s, MAX_CHAR);
    if (inFirstName) {
      n.Kanji = _tcstok_s(s, delimiter, &next);
      n.Hiragana = _tcstok_s(next, delimiter, &next);
      if (_tcstok_s(next, delimiter, &next)[0] == 
        _T('1')) {
        n.IsMale = true;
      } else {
        n.IsMale = false;
      }
      firstName.push_back(n);
    }
  }
  inFirstName.close();

  for (int i = 0; i < howMuchData; i++) {
    int lastNameIndex = 
      GetRandomNumber(lastName.size());
    int firstNameIndex = 
      GetRandomNumber(firstName.size());

    tcout << 
      FixLength << lastName[lastNameIndex].Hiragana <<
      FixLength << firstName[firstNameIndex].Hiragana <<
      FixLength << lastName[lastNameIndex].Kanji <<
      FixLength << firstName[firstNameIndex].Kanji << 
      firstName[firstNameIndex].IsMale << endl;
  }
  return result;
}

int GetRandomNumber(int max) {
  if (max < 1) return -1;
  unsigned int number;
  errno_t err = rand_s(&number);
  if (err != 0) {
    tcout << _T("rand_s֐s\n");
    return -1;
  } else {
    return (int) ((double) number / 
      (double) UINT_MAX * (double) max);
  }
}
